@@ -206,7 +206,7 @@ fn load_image(
     // Adobe Illustrator generate data: URLs without MIME-type for image
     // data.  We'll catch this and fall back to sniffing by unsetting the
     // content_type.
-    if content_type.as_ref().map(String::as_str) == Some("text/plain;charset=US-ASCII") {
+    if content_type.as_deref() == Some("text/plain;charset=US-ASCII") {
         content_type = None;
     }
 
@@ -227,8 +227,7 @@ fn load_image(
         None
     };
 
-    let surface =
-        SharedImageSurface::from_pixbuf(&pixbuf, bytes, content_type.as_ref().map(String::as_str))?;
+    let surface = SharedImageSurface::from_pixbuf(&pixbuf, bytes, content_type.as_deref())?;
 
     Ok(surface)
 }
@@ -312,10 +311,7 @@ impl<'i> AcquiredNodes<'i> {
         if node.borrow_element().is_accessed_by_reference() {
             self.acquire_ref(&node)
         } else {
-            Ok(AcquiredNode {
-                stack: None,
-                node: node.clone(),
-            })
+            Ok(AcquiredNode { stack: None, node })
         }
     }
 
@@ -353,7 +349,7 @@ impl NodeStack {
     }
 
     pub fn contains(&self, node: &Node) -> bool {
-        self.0.iter().find(|n| **n == *node).is_some()
+        self.0.iter().any(|n| *n == *node)
     }
 }
 
@@ -380,8 +376,8 @@ impl DocumentBuilder {
         type_: Option<String>,
         href: &str,
     ) -> Result<(), LoadingError> {
-        if type_.as_ref().map(String::as_str) != Some("text/css")
-            || (alternate.is_some() && alternate.as_ref().map(String::as_str) != Some("no"))
+        if type_.as_deref() != Some("text/css")
+            || (alternate.is_some() && alternate.as_deref() != Some("no"))
         {
             return Err(LoadingError::BadStylesheet);
         }
