@@ -311,9 +311,9 @@ impl DrawingCtx {
                         );
                     }
                 }
-                None // so that the following and_then() won't run
+                None
             })
-            .and_then(|t| {
+            .map(|t| {
                 self.cr.transform(t.into());
 
                 if let Some(vbox) = vbox {
@@ -321,10 +321,9 @@ impl DrawingCtx {
                         cr.rectangle(vbox.0.x0, vbox.0.y0, vbox.0.width(), vbox.0.height());
                         cr.clip();
                     }
-
-                    Some(self.push_view_box(vbox.0.width(), vbox.0.height()))
+                    self.push_view_box(vbox.0.width(), vbox.0.height())
                 } else {
-                    Some(self.get_view_params())
+                    self.get_view_params()
                 }
             })
     }
@@ -343,9 +342,14 @@ impl DrawingCtx {
         let units = borrow_element_as!(node, ClipPath).get_units();
 
         if let Ok(transform) = bbox.rect_to_transform(units) {
+            let node_transform = node
+                .borrow_element()
+                .get_transform()
+                .post_transform(&transform);
+
             let cascaded = CascadedValues::new_from_node(node);
 
-            self.with_saved_transform(Some(transform), &mut |dc| {
+            self.with_saved_transform(Some(node_transform), &mut |dc| {
                 let cr = dc.get_cairo_context();
 
                 // here we don't push a layer because we are clipping
@@ -355,11 +359,9 @@ impl DrawingCtx {
 
                 res
             })
-            .and_then(|_bbox|
-			  // Clipping paths do not contribute to bounding boxes (they should,
-			  // but we need Real Computational Geometry(tm), so ignore the
-			  // bbox from the clip path.
-			  Ok(()))
+            // Clipping paths do not contribute to bounding boxes (they should, but we
+            // need Real Computational Geometry(tm), so ignore the bbox from the clip path.
+            .map(|_bbox| ())
         } else {
             Ok(())
         }
@@ -508,7 +510,8 @@ impl DrawingCtx {
                 // Here we are clipping in user space, so the bbox doesn't matter
                 dc.clip_to_node(&clip_in_user_space, acquired_nodes, &dc.empty_bbox())?;
 
-                let needs_temporary_surface = !(opacity == 1.0
+                let is_opaque = (opacity - 1.0).abs() < f64::EPSILON;
+                let needs_temporary_surface = !(is_opaque
                     && filter.is_none()
                     && mask.is_none()
                     && clip_in_object_space.is_none());
@@ -601,12 +604,11 @@ impl DrawingCtx {
                                             &bbox,
                                             acquired_nodes,
                                         )
-                                        .and_then(|mask_surf| {
+                                        .map(|mask_surf| {
                                             if let Some(surf) = mask_surf {
                                                 dc.cr.set_matrix(affines.compositing.into());
                                                 dc.cr.mask_surface(&surf, 0.0, 0.0);
                                             }
-                                            Ok(())
                                         })
                                         .map(|_: ()| bbox)
                                     });
@@ -783,7 +785,7 @@ impl DrawingCtx {
                     }
                     _ => {
                         rsvg_log!(
-                            "element {} will not be rendered since \"{}\" is not a filter",
+                            "element {} will not be filtered since \"{}\" is not a filter",
                             node,
                             filter_uri,
                         );
@@ -792,7 +794,7 @@ impl DrawingCtx {
             }
             _ => {
                 rsvg_log!(
-                    "element {} will not be rendered since its filter \"{}\" was not found",
+                    "element {} will not be filtered since its filter \"{}\" was not found",
                     node,
                     filter_uri,
                 );
@@ -800,11 +802,7 @@ impl DrawingCtx {
         }
 
         // Non-existing filters must act as null filters (an empty surface is returned).
-        Ok(SharedImageSurface::empty(
-            child_surface.width(),
-            child_surface.height(),
-            child_surface.surface_type(),
-        )?)
+        Ok(child_surface)
     }
 
     fn set_color(
@@ -954,7 +952,7 @@ impl DrawingCtx {
                 &bbox,
                 current_color,
             )
-            .and_then(|had_paint_server| {
+            .map(|had_paint_server| {
                 if had_paint_server {
                     if values.stroke().0 == PaintServer::None {
                         cr.fill();
@@ -962,8 +960,6 @@ impl DrawingCtx {
                         cr.fill_preserve();
                     }
                 }
-
-                Ok(())
             })
             .and_then(|_| {
                 self.set_source_paint_server(
@@ -973,11 +969,10 @@ impl DrawingCtx {
                     &bbox,
                     current_color,
                 )
-                .and_then(|had_paint_server| {
+                .map(|had_paint_server| {
                     if had_paint_server {
                         cr.stroke();
                     }
-                    Ok(())
                 })
             });
 
@@ -985,7 +980,7 @@ impl DrawingCtx {
         // we leave it around from computing the bounding box
         cr.new_path();
 
-        res.and_then(|_: ()| Ok(bbox))
+        res.map(|_: ()| bbox)
     }
 
     pub fn draw_path(
@@ -1002,7 +997,8 @@ impl DrawingCtx {
                 self.with_discrete_layer(node, acquired_nodes, values, clipping, &mut |an, dc| {
                     let cr = dc.get_cairo_context();
 
-                    path.to_cairo(&cr)?;
+                    let is_square_linecap = values.stroke_line_cap() == StrokeLinecap::Square;
+                    path.to_cairo(&cr, is_square_linecap)?;
 
                     if clipping {
                         cr.set_fill_rule(cairo::FillRule::from(values.clip_rule()));
@@ -1298,14 +1294,14 @@ fn get_clip_in_user_and_object_space(
                 .ok()
                 .filter(|a| is_element_of_type!(*a.get(), ClipPath))
         })
-        .and_then(|acquired| {
+        .map(|acquired| {
             let clip_node = acquired.get().clone();
 
             let units = borrow_element_as!(clip_node, ClipPath).get_units();
 
             match units {
-                CoordUnits::UserSpaceOnUse => Some((Some(clip_node), None)),
-                CoordUnits::ObjectBoundingBox => Some((None, Some(clip_node))),
+                CoordUnits::UserSpaceOnUse => (Some(clip_node), None),
+                CoordUnits::ObjectBoundingBox => (None, Some(clip_node)),
             }
         })
         .unwrap_or((None, None))
