@@ -14,6 +14,7 @@ use crate::document::{AcquiredNodes, NodeStack};
 use crate::drawing_ctx::{DrawingCtx, ViewParams};
 use crate::element::{Draw, Element, ElementResult, SetAttributes};
 use crate::error::*;
+use crate::href::{is_href, set_href};
 use crate::length::*;
 use crate::node::{CascadedValues, Node, NodeBorrow};
 use crate::paint_server::{AsPaintSource, PaintSource};
@@ -95,15 +96,13 @@ fn validate_offset(length: Length<Both>) -> Result<Length<Both>, ValueErrorKind>
 
 impl SetAttributes for Stop {
     fn set_attributes(&mut self, pbag: &PropertyBag<'_>) -> ElementResult {
-        for (attr, value) in pbag.iter() {
-            match attr.expanded() {
-                expanded_name!("", "offset") => {
-                    self.offset = attr
-                        .parse_and_validate(value, validate_offset)
-                        .map(|l| UnitInterval::clamp(l.length))?
-                }
-                _ => (),
-            }
+        let result = pbag
+            .iter()
+            .find(|(attr, _)| attr.expanded() == expanded_name!("", "offset"))
+            .and_then(|(attr, value)| attr.parse_and_validate(value, validate_offset).ok())
+            .map(|l| UnitInterval::clamp(l.length));
+        if let Some(offset) = result {
+            self.offset = offset
         }
 
         Ok(())
@@ -598,8 +597,12 @@ impl SetAttributes for Common {
                     self.transform = Some(attr.parse(value)?)
                 }
                 expanded_name!("", "spreadMethod") => self.spread = Some(attr.parse(value)?),
-                expanded_name!(xlink "href") => {
-                    self.fallback = Some(Fragment::parse(value).attribute(attr)?)
+                ref a if is_href(a) => {
+                    set_href(
+                        a,
+                        &mut self.fallback,
+                        Fragment::parse(value).attribute(attr.clone())?,
+                    );
                 }
                 _ => (),
             }
@@ -773,7 +776,9 @@ impl Gradient {
     ) {
         let transform = transform.pre_transform(&self.transform);
 
-        transform.invert().map(|m| pattern.set_matrix(m.into()));
+        if let Some(m) = transform.invert() {
+            pattern.set_matrix(m.into())
+        }
         pattern.set_extend(cairo::Extend::from(self.spread));
         self.add_color_stops_to_pattern(pattern, opacity);
     }
