@@ -13,6 +13,7 @@ use crate::drawing_ctx::{DrawingCtx, ViewParams};
 use crate::element::{Draw, Element, ElementResult, SetAttributes};
 use crate::error::*;
 use crate::float_eq_cairo::ApproxEqCairo;
+use crate::href::{is_href, set_href};
 use crate::length::*;
 use crate::node::{CascadedValues, Node, NodeBorrow, NodeDraw, WeakNode};
 use crate::paint_server::{AsPaintSource, PaintSource};
@@ -133,8 +134,12 @@ impl SetAttributes for Pattern {
                 expanded_name!("", "patternTransform") => {
                     self.common.affine = Some(attr.parse(value)?)
                 }
-                expanded_name!(xlink "href") => {
-                    self.fallback = Some(Fragment::parse(value).attribute(attr)?);
+                ref a if is_href(a) => {
+                    set_href(
+                        a,
+                        &mut self.fallback,
+                        Fragment::parse(value).attribute(attr.clone())?,
+                    );
                 }
                 expanded_name!("", "x") => self.common.x = Some(attr.parse(value)?),
                 expanded_name!("", "y") => self.common.y = Some(attr.parse(value)?),
@@ -368,12 +373,14 @@ impl AsPaintSource for ResolvedPattern {
         // Set the final surface as a Cairo pattern into the Cairo context
         let pattern = cairo::SurfacePattern::create(&surface);
 
-        affine.invert().map(|m| pattern.set_matrix(m.into()));
+        if let Some(m) = affine.invert() {
+            pattern.set_matrix(m.into())
+        }
         pattern.set_extend(cairo::Extend::Repeat);
         pattern.set_filter(cairo::Filter::Best);
         cr_save.set_source(&pattern);
 
-        res.and_then(|_| Ok(true))
+        res.map(|_| true)
     }
 }
 
