@@ -11,6 +11,7 @@ use crate::dpi::Dpi;
 use crate::drawing_ctx::{ClipMode, DrawingCtx, ViewParams};
 use crate::element::{Draw, ElementResult, SetAttributes};
 use crate::error::*;
+use crate::href::{is_href, set_href};
 use crate::length::*;
 use crate::node::{CascadedValues, Node, NodeBorrow, NodeDraw};
 use crate::parsers::{Parse, ParseValue};
@@ -242,13 +243,11 @@ impl Draw for Svg {
 
         let is_measuring_toplevel_svg = !has_parent && draw_ctx.is_measuring();
 
-        let (viewport, vbox) = if is_measuring_toplevel_svg {
+        let (viewport, vbox) = if is_measuring_toplevel_svg || has_parent {
             // We are obtaining the toplevel SVG's geometry.  This means, don't care about the
             // DrawingCtx's viewport, just use the SVG's intrinsic dimensions and see how far
             // it wants to extend.
             (svg_viewport, self.vbox)
-        } else if has_parent {
-            (svg_viewport, self.vbox)
         } else {
             (
                 // The client's viewport overrides the toplevel's x/y/w/h viewport
@@ -309,9 +308,12 @@ impl SetAttributes for Use {
     fn set_attributes(&mut self, pbag: &PropertyBag<'_>) -> ElementResult {
         for (attr, value) in pbag.iter() {
             match attr.expanded() {
-                expanded_name!(xlink "href") => {
-                    self.link = Some(Fragment::parse(value).attribute(attr)?)
-                }
+                ref a if is_href(a) => set_href(
+                    a,
+                    &mut self.link,
+                    Fragment::parse(value).attribute(attr.clone())?,
+                ),
+
                 expanded_name!("", "x") => self.x = attr.parse(value)?,
                 expanded_name!("", "y") => self.y = attr.parse(value)?,
                 expanded_name!("", "width") => {
@@ -394,11 +396,12 @@ impl ClipPath {
 
 impl SetAttributes for ClipPath {
     fn set_attributes(&mut self, pbag: &PropertyBag<'_>) -> ElementResult {
-        for (attr, value) in pbag.iter() {
-            match attr.expanded() {
-                expanded_name!("", "clipPathUnits") => self.units = attr.parse(value)?,
-                _ => (),
-            }
+        let result = pbag
+            .iter()
+            .find(|(attr, _)| attr.expanded() == expanded_name!("", "clipPathUnits"))
+            .and_then(|(attr, value)| attr.parse(value).ok());
+        if let Some(units) = result {
+            self.units = units
         }
 
         Ok(())
@@ -489,7 +492,7 @@ impl SetAttributes for Link {
     fn set_attributes(&mut self, pbag: &PropertyBag<'_>) -> ElementResult {
         for (attr, value) in pbag.iter() {
             match attr.expanded() {
-                expanded_name!(xlink "href") => self.link = Some(value.to_owned()),
+                ref a if is_href(a) => set_href(a, &mut self.link, value.to_owned()),
                 _ => (),
             }
         }
